using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;

namespace PropertyNamesGenerator.Parsers
{
    public class HbmRegexParser : IFileParser
    {

        public static ClassDictionary ParseDocument(string fullPathFileName)
        {
            #region Guard
            
            if (!File.Exists(fullPathFileName))
                throw new FileNotFoundException("Could not find file", fullPathFileName);
            #endregion

            ClassDictionary mainclass = new ClassDictionary();
            ClassDictionary subclasses = new ClassDictionary();

            string[] fileLines = File.ReadAllLines(fullPathFileName);

            string currentClass = "";
            string currentSubClass = "";

            foreach (string line in fileLines)
            {
                if (IsClass(line))
                {
                    string className = GetClassName(line, ClassType.Class);
                    if (!mainclass.ContainsKey(className))
                        mainclass.Add(className, new List<string>());

                    currentClass = className;
                }
                if (IsSubClass(line))
                {
                    string subclassName = GetClassName(line, ClassType.SubClass);
                    if (!subclasses.ContainsKey(subclassName))
                        subclasses.Add(subclassName, new List<string>());

                    currentClass = "";
                    currentSubClass = subclassName;
                }
                if (IsProperty(line))
                {
                    string propertyName = GetPropertyName(line);
                    if (string.IsNullOrEmpty(currentClass))
                        subclasses[currentSubClass].Add(propertyName);
                    else
                        mainclass[currentClass].Add(propertyName);
                }
            }

            //Add properties from hierarchy to subclasses
            if (subclasses.Count > 0)
            {
                if (mainclass.Keys.Count > 1)
                    throw new Exception("HbmRegexParser only supports HBM-files with one main class when subclasses are present.");

                AddBaseClassPropertiesToSubclasses(mainclass, subclasses);
            }

            //Add all subclasses to main class list
            foreach (string key in subclasses.Keys)
            {
                mainclass.Add(key, subclasses[key]);
            }
            
            return mainclass;
        }

        /// <summary>
        /// Add the base class properties to each of the subclasses
        /// </summary>
        /// <param name="baseclass">base class</param>
        /// <param name="subclasses">subclasses</param>
        private static void AddBaseClassPropertiesToSubclasses(ClassDictionary baseclass, ClassDictionary subclasses)
        {
            foreach (List<string> properties in baseclass.Values)
            {
                foreach (string property in properties)
                {
                    foreach (string key in subclasses.Keys)
                    {
                        subclasses[key].Add(property);
                    }
                }
            }
        }

        private static bool IsClass(string line)
        {
            string propertyMatchPattern = @".*\<class .*";

            return Regex.IsMatch(line, propertyMatchPattern);
        }

        private static bool IsSubClass(string line)
        {
            string propertyMatchPattern = @".*\<subclass .*";

            return Regex.IsMatch(line, propertyMatchPattern);
        }

        private static bool IsProperty(string line)
        {
            string propertyMatchPattern = @".*\<property .*";

            return Regex.IsMatch(line, propertyMatchPattern);
        }

        private static string GetClassName(string line, ClassType classType)
        {
            string className = "";

            string classNameIdentifier = "";
            if (classType == ClassType.Class)
                classNameIdentifier = "class";
            else
                classNameIdentifier = "subclass";

            string classNameRetrievalPattern = string.Format(@".*\<{0}.*name\=\""(?<ClassName>[^\""]*)", classNameIdentifier);

            Match match = Regex.Match(line, classNameRetrievalPattern);
            if (match.Success)
            {
                string fullClassName = match.Groups["ClassName"].Value;

                className = fullClassName.Substring(fullClassName.LastIndexOf(".") + 1,
                                                    fullClassName.LastIndexOf(",") - fullClassName.LastIndexOf(".") - 1);
            }
            return className;
        }

        private static string GetPropertyName(string line)
        {
            string propertyName = "";
            string propertyNameRetrievalPattern = @".*\<property.*name\=\""(?<PropertyName>[^\""]*)";

            Match match = Regex.Match(line, propertyNameRetrievalPattern);
            if (match.Success)
            {
                propertyName = match.Groups["PropertyName"].Value;
            }
            return propertyName;
        }    
    }
}
