#region license
// Copyright (c) 2008 Rune Sundling (rune.sundling@gmail.com)
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
// 
//     * Redistributions of source code must retain the above copyright notice and this list of conditions.
//     * Redistributions in binary form must reproduce the above copyright notice 
//      and this list of conditions in the documentation and/or other materials provided with the distribution.
#endregion
using System;
using PropertyNamesGenerator.Util;

namespace PropertyNamesGenerator
{
    public class Program
    {
        static void Main(string[] args)
        {
            Run(args);
        }

        public static void Run(string[] args)
        {
            string inputDir = "";
            string searchString = "";
            string outputDir = "";
            Language language = Language.CSharp;

            if (args.Length == 0)
            {
                Output.WriteHelpMessage();
            }

            foreach (string arg in args)
            {
                if (arg.Contains("/files:"))
                {
                    inputDir = GetInputDir(arg);
                    searchString = GetSearchString(arg);

                    Output.NotifyUser(
                        string.Format("Found files-argument with search directory {0}, and search string {1}", inputDir,
                                      searchString),
                        OutputLevel.Info);
                }
                else if (arg.Contains("/out:"))
                {
                    outputDir = GetOutputDir(arg);
                    
                    Output.NotifyUser(string.Format("Found out-argument with output directory {0}", outputDir),
                                      OutputLevel.Info);
                }
                else if (arg.Contains("/lang:"))
                {
                    language = ParseLanguage(arg);
                }
                else if (arg.ToLower().Contains("-v"))
                {
                    //Verbose mode
                    Output.SetOutputLevelTrigger(OutputLevel.Info);
                }
                else if (arg.Contains("-?") || arg.Contains("-help"))
                {
                    Output.WriteHelpMessage();
                    return;
                }
            }

            if (!Validator.ValidateProgramInput(inputDir, searchString, outputDir))
                return;

            Generator.GeneratePropertyNameClasses(inputDir, searchString, outputDir, language);
        }

        #region Input-parsing


        private static Language ParseLanguage(string arg)
        {
            Language language;
            arg = arg.Replace("/lang:", "")
                    .Replace("\"", "");
            
            switch (arg.ToUpper())
            {   
                case "CS":
                {
                    language = Language.CSharp;
                    break;
                }
                case "VB":
                {
                    language = Language.VB;
                    break;
                }
                default:
                {
                    throw new Exception(
                        string.Format("Language identifier {0} was not identified as a valid language. ", arg));
                }
            }

            return language;
    }

        /// <summary>
        /// Retrieve input directory from /files: argument
        /// </summary>
        /// <param name="arg">files-argument</param>
        /// <returns>input directory</returns>
        private static string GetInputDir(string arg)
        {
            arg = arg.Replace("/files:", "")
                    .Replace("\"", "");
            return arg.Substring(0, arg.LastIndexOf("/"));
        }

        /// <summary>
        /// Retrieve output directory from /out: argument
        /// </summary>
        /// <param name="arg">out-argument</param>
        /// <returns>output directory</returns>
        private static string GetOutputDir(string arg)
        {
            return (
                arg.Replace("/out:", "")
                    .Replace("\"", "")
                    );
        }
        
        /// <summary>
        /// Retrieve search string from /files: argument
        /// </summary>
        /// <param name="arg">files-argument</param>
        /// <returns>search string</returns>
        private static string GetSearchString(string arg)
        {
            return arg.Substring(arg.LastIndexOf("/") + 1);
        }

        #endregion


    }
}